'''
Module with utility functions for reading and writting files
'''

# -----------------------
def _format(line : str) -> str:
    line_lstr = line.lstrip()
    if line_lstr.startswith('-'):
        ind=line.index('-')
        return line[:ind-1] + '   -' + line[ind+1:]

    return line
# -----------------------
def reformat_yaml(path : str) -> None:
    '''
    Picks up path to yaml file and improves formatting
    Needed because PyYaml does not indent lists
    '''

    with open(path, encoding='utf-8') as ifile:
        l_line = ifile.read().splitlines()

    l_line = [ _format(line) for line in l_line ]

    text = '\n'.join(l_line)
    with open(path, 'w', encoding='utf-8') as ofile:
        ofile.write(text)
# -----------------------



'''
This script will do checks on the rd_ap_2024 before pushing to feature branch

This problems will be caught before pipelines run
'''
import re

from typing              import Union
from collections         import Counter
from importlib.resources import files

import argparse
import yaml

import ap_utilities.io.utilities     as iout 
import ap_utilities.decays.utilities as aput
from ap_utilities.logging.log_store import LogStore

log = LogStore.add_logger('ap_utilities:check_production')
# --------------------------
class Data:
    '''
    Class storing shared attributes
    '''
    analysis    : str
    prod_path   : str
    regex_info  : str = r'"([\w,_,.,-]+)"'
    d_samples   : dict[str, set[str]]            = {}
    l_info      : list[list[str]]                = []
    d_report    : dict                           = {'missing' : {}}
# --------------------------
def _parse_args() -> None:
    parser = argparse.ArgumentParser(description='')
    parser.add_argument('-p', '--prod_path', type=str , help='Path to directory with production, rd_ap_2024', required= True)
    parser.add_argument('-a', '--analysis' , type=str , help='Type of analysis for which to check samples'  , required= True, choices=['rx', 'rk'])
    args = parser.parse_args()

    Data.prod_path = args.prod_path
    Data.analysis  = args.analysis
# -------------------------
def _load_yaml(name : str) -> dict:
    path = f'{Data.prod_path}/{name}'
    with open(path, encoding='utf-8') as ifile:
        d_data = yaml.safe_load(ifile)

    return d_data
# -------------------------
def _get_lines(name : str) -> list[str]:
    path = f'{Data.prod_path}/{name}'
    with open(path, encoding='utf-8') as ifile:
        l_line = ifile.read().splitlines()

    return l_line
# -------------------------
def _sample_from_line(line : str) -> Union[None,list[str]]:
    '''
    Takes line from info.yaml
    Returns a list of strings, each with configuration for an MC sample, if line is not present, returns None
    '''
    l_match = re.findall(Data.regex_info, line)
    if len(l_match) != 10:
        return None

    return l_match
# -------------------------
def _sample_from_info(l_info_line : list[str]) -> list[list[str]]:
    '''
    Takes list of lines in info.yaml
    Returns list of sets of strings defining MC samples
    '''
    l_retval   = [ _sample_from_line(line) for line     in l_info_line                        ]
    l_l_config = [ l_config                for l_config in l_retval    if l_config is not None]

    return l_l_config
# -------------------------
def _samples_from_info_lines( l_line : list[str]) -> list[str]:
    l_retval  = [ _sample_from_line(line) for line   in l_line                        ]
    l_sample  = [ retval[0]               for retval in l_retval if retval is not None]

    return l_sample
# -------------------------
def _print_repeated(l_line : list[str]) -> None:
    counter = Counter(l_line)
    l_repeated = [ (element, count) for element, count in counter.items() if count > 1 ]

    for repeated, count in l_repeated:
        log.info(f'{repeated:<40}{count:<10}')
# -------------------------
def _list_to_set(l_line : list[str], msg_repeated : Union[None,str]=None) -> set[str]:
    s_line = set(l_line)
    nlist  = len(l_line)
    nset   = len(s_line)

    if nlist != nset and msg_repeated is not None:
        log.error('Repeated elements:')
        _print_repeated(l_line)
        raise ValueError(msg_repeated)

    return s_line
# -------------------------
def _load_samples() -> None:
    l_info_line   = _get_lines('info.yaml')
    Data.l_info   = _sample_from_info(l_info_line)

    l_info_sample = _samples_from_info_lines(l_info_line)
    s_info_sample = _list_to_set(l_info_sample)

    d_mcdt        = _load_yaml('tupling/config/mcfuntuple.yaml')
    l_mcdt_sample = list(d_mcdt)
    s_mcdt_sample = _list_to_set(l_mcdt_sample, msg_repeated='Found repeated entries in mcfuntuple')

    d_samp        = _load_yaml('tupling/config/samples_turbo_lines_mapping.yaml')
    l_samp_sample = list(d_samp)
    s_samp_sample = _list_to_set(l_samp_sample, msg_repeated='Found repeated entries in samples.yaml')

    Data.d_samples['info'       ] = s_info_sample
    Data.d_samples['mcfuntuple' ] = s_mcdt_sample
    Data.d_samples['samples'    ] = s_samp_sample
    Data.d_samples[Data.analysis] = _get_analysis_nicknames()
# -------------------------
def _nickname_from_evt_type(evt_type : int) -> str:
    etype    = str(evt_type)
    nickname = aput.read_decay_name(event_type=etype, style= 'safe_1')

    return nickname
# -------------------------
def _flatten_list(lst : list) -> list:
    l_val = []
    for item in lst:
        if isinstance(item, list):
            l_val.extend(_flatten_list(item))
        else:
            l_val.append(item)

    return l_val
# -------------------------
def _get_analysis_nicknames() -> set[str]:
    evt_path = files('ap_utilities_data').joinpath('analyses.yaml')
    evt_path = str(evt_path)

    with open(evt_path, encoding='utf-8') as ifile:
        d_analysis = yaml.safe_load(ifile)

    l_evt_type = d_analysis[Data.analysis]
    l_evt_type = _flatten_list(l_evt_type)
    l_nick_name= [ _nickname_from_evt_type(event_type) for event_type in l_evt_type ]

    return set(l_nick_name)
# -------------------------
def _get_difference(s_val1 : set[str], s_val2 : set[str]) -> list[str]:
    s_diff = s_val1 - s_val2
    l_diff = list(s_diff)
    l_diff.sort()

    return l_diff
# -------------------------
def _check_samples(name_1 : str, name_2 : str) -> None:
    s_sample_1 = Data.d_samples[name_1]
    s_sample_2 = Data.d_samples[name_2]

    d_sample   = {}
    if s_sample_1 != s_sample_2:
        log.warning(f'Samples in {name_1} and {name_2} are different')

        d_sample[f'only {name_1}'] = _get_difference(s_sample_1, s_sample_2)
        d_sample[f'only {name_2}'] = _get_difference(s_sample_2, s_sample_1)

    Data.d_report['missing'][f'{name_1}_{name_2}'] = d_sample
# -------------------------
def _compare_groups() -> None:
    _check_samples('info'      ,  'mcfuntuple')
    _check_samples('info'      ,  'samples'   )
    _check_samples('mcfuntuple',  'samples'   )
    # Check that for chosen analysis, all the samples have been added
    _check_samples('info'      , Data.analysis)
# -------------------------
def _save_report():
    out_path = 'report.yaml'

    with open(out_path, 'w', encoding='utf-8') as ofile:
        yaml.safe_dump(Data.d_report, ofile, width=200)

    iout.reformat_yaml(path = out_path)
# -------------------------
def _check_name_lenghts() -> None:
    l_long_nickname = []
    for [nickname, evt_type, mc_path, polarity, _, _, _, nuval, sim_version, generator] in Data.l_info:
        mc_path  = mc_path.replace('.', '_')
        job_name = f'MC_{mc_path}_{polarity}_{nuval}_{sim_version}_{generator}_{evt_type}_{nickname}'
        size     = len(job_name)
        if size > 100:
            log.warning(f'{size:<20}{nickname:<100}')
            size = str(size)
            l_long_nickname.append([nickname, size])

    Data.d_report['long_nicknames'] = l_long_nickname
# -------------------------
def main():
    '''
    Start of execution
    '''
    _parse_args()
    _load_samples()
    _compare_groups()
    _check_name_lenghts()

    _save_report()
# -------------------------
if __name__ == '__main__':
    main()



'''
Script used to create YAML file with dictionary between Renato's nicknames and DecFiles based nicknames
'''
import re
import ap_utilities.physics.utilities as aput

# ----------------------------------
class Nicknames:
    '''
    Class used to create a dictionary between old naming and DecayFiles based naming
    '''
    def __init__(self, path : str):
        self._path  = path
        self._regex = r'#?\s*\(\s*(\".*\")\s*,\s*\"(.*)\"\s*,\s*"2024.*\)'
    # ----------------------------------
    def _is_good_line(self, line : str) -> bool:
        if 'ddb-' in line and 'sim' in line:
            return True

        return False
    # ----------------------------------
    def _get_lines(self) -> list[str]:
        with open(self._path, encoding='utf-8') as ifile:
            l_line = ifile.read().splitlines()

        l_line = [ line.replace('\'', '\"') for line in l_line ]
        l_line = [ line for line in l_line if self._is_good_line(line)]

        return l_line
    # ----------------------------------
    def _info_from_line(self, line : str) -> tuple[str, str]:
        mtch = re.match(self._regex, line)
        if not mtch:
            raise ValueError(f'Cannot match {line} with {self._regex}')

        name = mtch.group(1)
        evtt = mtch.group(2)

        name = name.replace('"', '')

        return name, evtt
    # ----------------------------------
    def _get_dictionary(self, l_line : list[str]) -> dict[str,str]:
        d_data = {}
        for line in l_line:
            name, evt_type = self._info_from_line(line)
            nick_name      = aput.read_decay_name(event_type=evt_type, style= 'safe_1')

            d_data[name] = nick_name

        return d_data
    # ----------------------------------
    def get_nicknames(self) -> dict[str,str]:
        '''
        Returns dictionary between old and new naming
        '''
        l_line = self._get_lines()
        d_nick = self._get_dictionary(l_line)

        return d_nick
# ----------------------------------



'''
Script with tests for LogInfo class
'''
import pytest

from ap_utilities.logfiles.log_info import LogInfo

# ----------------------------
class Data:
    '''
    Class storing shared data
    '''
    log_mcdt            = '/home/acampove/cernbox/dev/tests/ap_utilities/log_info/mcdt.zip'

    l_log_fallback_noline = [
    ('/home/acampove/cernbox/dev/tests/ap_utilities/log_info/noline.zip'         , 'Xib_psi2SXi_ee_Lambdapi_eq_TightCut'         , 13603),
    ('/home/acampove/cernbox/dev/tests/ap_utilities/log_info/fall_back_omega.zip', 'Omegab_JpsiOmega_mm_LambdaK_eq_phsp_TightCut', 13998),
    ]
# ----------------------------
def test_mcdt():
    '''
    Tests if the statistics used for MCDecayTree are read correctly
    '''
    obj = LogInfo(zip_path = Data.log_mcdt)
    nentries = obj.get_mcdt_entries('Bu_Kee_eq_btosllball05_DPC')

    assert nentries == 13584
# ----------------------------
def test_fallback_nofile():
    '''
    Tests if the statistics used for MCDecayTree are read correctly
    '''
    obj = LogInfo(zip_path = '/path/that/does/not/exist.zip')
    nentries = obj.get_mcdt_entries('Bu_Kee_eq_btosllball05_DPC')

    assert nentries == -1
# ----------------------------
@pytest.mark.parametrize('zip_path, sample, entries', Data.l_log_fallback_noline)
def test_fallback_noline(zip_path : str, sample : str, entries : int):
    '''
    Tests if the statistics used for MCDecayTree are read correctly
    '''
    obj = LogInfo(zip_path = zip_path)
    nentries = obj.get_mcdt_entries(sample)

    assert nentries == entries



'''
Script used to check which MC samples are found in grid
'''
import argparse

from dataclasses                          import dataclass

import yaml
from ap_utilities.logging.log_store       import LogStore
from ap_utilities.bookkeeping.bkk_checker import BkkChecker

log=LogStore.add_logger('ap_utilities_scripts:check_samples')
# --------------------------------
@dataclass
class Data:
    '''
    Class storing shared attributes
    '''

    input_path  : str
    nthread     : int
    log_lvl     : int
# ----------------------------------------
def _sections_from_path(path : str) -> dict[str, dict]:
    with open(path, encoding='utf-8') as ifile:
        d_cfg = yaml.safe_load(ifile)

    return d_cfg
# --------------------------------
def _parse_args() -> None:
    parser = argparse.ArgumentParser(description='Used to filter samples based on what exists in the GRID')
    parser.add_argument('-i', '--input'  , type=str, help='Path to input YAML file')
    parser.add_argument('-n', '--nthread', type=int, help='Number of threads', default=1)
    parser.add_argument('-l', '--log_lvl', type=int, help='Logging level', default=20, choices=[10,20,30,40])
    args = parser.parse_args()

    Data.input_path  = args.input
    Data.nthread     = args.nthread
    Data.log_lvl     = args.log_lvl
# --------------------------------
def _set_logs() -> None:
    log.debug(f'Running with log level: {Data.log_lvl}')

    LogStore.set_level('ap_utilities:Bookkeeping.bkk_checker', Data.log_lvl)
    LogStore.set_level('ap_utilities_scripts:check_samples'  , Data.log_lvl)
# --------------------------------
def main():
    '''
    Script starts here
    '''
    _parse_args()
    _set_logs()

    d_cfg      = _sections_from_path(Data.input_path)
    d_sections = d_cfg['sections']
    for name, d_section in d_sections.items():
        log.info(f'Processing section: {name}')
        obj=BkkChecker(name, d_section)
        obj.save(nthreads=Data.nthread)
# --------------------------------
if __name__ == '__main__':
    main()



'''
Module containing utility functions
'''
from functools           import cache
from importlib.resources import files

import yaml

# ---------------------------------
class Data:
    '''
    Class storing attributes shared
    '''

    d_form_long : dict[str,str]
    d_form_short: dict[str,str]

    is_initialized = False
# ---------------------------------
def _initialize():
    if Data.is_initialized:
        return

    d_form_short                        = {}
    d_form_short[                  '.'] =      'p'
    d_form_short[                  '-'] =     'mn'
    d_form_short[                  '+'] =     'pl'
    d_form_short[                  '='] =   '_eq_'
    d_form_short[                  ','] =      '_'
    d_form_short['GeV'                ] =      'G'

    d_form_long                         = {}
    d_form_long [         'DecProdCut'] =    'DPC'
    d_form_long [ 'EvtGenDecayWithCut'] =  'EGDWC'
    d_form_long ['VisibleInAcceptance'] =    'VIA'
    d_form_long [        'HighVisMass'] =    'HVM'
    d_form_long [       'OppositeSign'] =     'OS'
    d_form_long [           'TightCut'] =     'TC'
    d_form_long ['DiMuon_OppositeSign'] = 'DiM_OS'

    Data.d_form_long = d_form_long
    Data.d_form_short= d_form_short

    Data.is_initialized = True
# ---------------------------------
def _apply_format(d_format : dict[str,str], name : str) -> str:
    for org, new in d_format.items():
        name = name.replace(org, new)

    return name
# ---------------------------------
@cache
def _load_data(file_name : str) -> dict:
    file_path = files('ap_utilities_data').joinpath(file_name)
    file_path = str(file_path)
    with open(file_path, encoding='utf-8') as ifile:
        d_data = yaml.safe_load(ifile)

    return d_data
# ---------------------------------
def format_nickname(nickname : str) -> str:
    '''
    Function taking decays nickname and returning formatted version

    nickaname: Name to be formatted
    '''
    _initialize()

    nickname = _apply_format(Data.d_form_long , nickname)
    nickname = _apply_format(Data.d_form_short, nickname)

    return nickname
# ---------------------------------
# ---------------------------------
def read_decay_name(event_type : str, formatted : bool = True) -> str:
    '''
    Takes event type, and style strings, returns nickname of decay as defined in DecFiles package

    Styles:

    literal   : No change is made to nickname
    formatted : If true, will return name after formatting 
    '''
    _initialize()

    yaml_path  = 'evt_form.yaml' if formatted else 'evt_name.yaml'

    d_evt_name = _load_data(yaml_path)

    if event_type not in d_evt_name:
        raise ValueError(f'Event type {event_type} not found')

    value = d_evt_name[event_type]

    return value
# ---------------------------------
def read_event_type(nickname : str) -> str:
    '''
    Takes nickname after reformatting, i.e. replacement of commans, equals, etc.
    Returns corresponding event type 
    '''
    _initialize()

    d_name_evt = _load_data('form_evt.yaml')

    if nickname not in d_name_evt:
        raise ValueError(f'Event type {nickname} not found')

    value = d_name_evt[nickname]

    return value
# ---------------------------------
def new_from_old_nick(nickname : str) -> str:
    '''
    Function that takes a decay nick name using Run1/2 naming
    and returns nicknames using Run3 naming
    '''
    _initialize()

    d_old_evt = _load_data('old_name_evt.yaml')
    if nickname not in d_old_evt:
        raise ValueError(f'Old nickname {nickname} not found in: old_name_evt.yaml')

    evt_type   = d_old_evt[nickname]

    d_evt_name = _load_data('evt_form.yaml')
    if evt_type not in d_evt_name:
        raise ValueError(f'Event type {evt_type} not found in: evt_name.yaml')

    new_nick   = d_evt_name[evt_type]

    return new_nick
# ---------------------------------
def old_from_new_nick(nickname : str) -> str:
    '''
    Function that takes a decay nick name using Run3 naming
    and returns nicknames using Run1/2 naming
    '''
    _initialize()

    d_name_evt = _load_data('form_evt.yaml')
    if nickname not in d_name_evt:
        raise ValueError(f'Nickname {nickname} not found in: name_evt.yaml')

    evt_type   = d_name_evt[nickname]

    d_evt_old  = _load_data('evt_old_name.yaml')
    if evt_type not in d_evt_old:
        raise ValueError(f'Event type {evt_type} not found in: evt_old_name.yaml')

    old_nick   = d_evt_old[evt_type]

    return old_nick
# ---------------------------------
def name_from_lower_case(lower_case : str) -> str:
    '''
    Using new naming, but all lower case, will return
    original naming. Needed to deal with way AP names samples.
    '''
    _initialize()

    d_data  = _load_data('lower_original.yaml')
    org_arg = lower_case

    if   lower_case.endswith('gev'):
        lower_case = lower_case[:-2]
    elif lower_case.endswith('_spr'):
        lower_case = lower_case[:-4]
    elif lower_case.endswith('_ss'):
        lower_case = lower_case[:-3]
    else:
        pass

    if lower_case not in d_data:
        raise ValueError(f'Sample {lower_case} not found in: lower_original.yaml')

    name = d_data[lower_case]
    if org_arg.endswith('_ss'):
        name = f'{name}_SS'

    return name
# ---------------------------------



'''
Module storing LogInfo class
'''
import os
import re
import glob
import zipfile
import functools
from typing import Union

from ap_utilities.logging.log_store import LogStore

log = LogStore.add_logger('ap_utilities:log_info')
# ---------------------------------------------
class LogInfo:
    '''
    Class taking a zip file with logging information from AP pipelines
    and extracting information like the number of entries that it ran over
    '''
    # ---------------------------------------------
    def __init__(self, zip_path : str):
        self._zip_path = zip_path
        self._log_wc   = 'DaVinci_*.log'

        self._out_path : str
        self._log_path : str

        self._entries_regex : str = r'\s*\|\s*"#\snon-empty events for field .*"\s*\|\s*(\d+)\s*\|.*'
    # ---------------------------------------------
    def _get_log_path(self) -> str:
        path_wc = f'{self._out_path}/*/{self._log_wc}'

        try:
            [log_path] = glob.glob(path_wc)
        except ValueError as exc:
            raise FileNotFoundError(f'Cannot find one and only one DaVinci log file in: {path_wc}') from exc

        return log_path
    # ---------------------------------------------
    @functools.lru_cache()
    def _get_dv_lines(self) -> Union[list[str],None]:
        if not os.path.isfile(self._zip_path):
            log.warning(f'Cannot find: {self._zip_path}')
            return None

        with zipfile.ZipFile(self._zip_path, 'r') as zip_ref:
            zip_ref.extractall(self._out_path)

        self._log_path = self._get_log_path()

        with open(self._log_path, encoding='utf-8') as ifile:
            l_line = ifile.read().splitlines()

        return l_line
    # ---------------------------------------------
    def _entries_from_line(self, line : str) -> Union[int,None]:
        mtch = re.match(self._entries_regex, line)
        if not mtch:
            log.warning(f'Cannot extract number of entries from line \"{line}\" using regex \"{self._entries_regex}\"')
            return None

        entries = mtch.group(1)

        return int(entries)
    # ---------------------------------------------
    def _all_in_line(self, line : str, l_substr : list[str]) -> bool:
        found = True
        for substr in l_substr:
            found = found and substr in line

        return found
    # ---------------------------------------------
    def _index_at_first_instance(self, l_line : list[str], l_substr : list[str]) -> Union[int,None]:
        index = None
        for i_line, line in enumerate(l_line):
            if self._all_in_line(line, l_substr):
                index = i_line
                break

        return index
    # ---------------------------------------------
    def _get_line_with_entries(self, l_line : list[str], alg_name : str) -> Union[str,None]:
        algo_index = self._index_at_first_instance(l_line, l_substr=[alg_name, 'Number of counters'])
        if algo_index is None:
            log.warning(f'Cannot find line with \"Number of counters\" and \"{alg_name}\" in {self._log_path}')
            return None

        l_line = l_line[algo_index:]

        atrm_index = self._index_at_first_instance(l_line, l_substr=['ApplicationMgr'])
        if atrm_index is None:
            log.warning('Cannot find line with ApplicationMgr')
            return None

        l_line = l_line[:atrm_index]

        none_index = self._index_at_first_instance(l_line, l_substr=[' | "# non-empty events for field'])
        if none_index is None:
            log.warning('Cannot find line with non empty events line')
            return None

        return l_line[none_index]
    # ---------------------------------------------
    def get_mcdt_entries(self, alg_name : str, fall_back : int = -1) -> int:
        '''
        Returns entries that DaVinci ran over to get MCDecayTree
        '''
        # If not clipped, long names will cause failure
        # due to clipping in logs
        alg_name       = alg_name[:30]
        self._out_path = f'/tmp/log_info/{alg_name}'
        os.makedirs(self._out_path, exist_ok=True)

        l_line            = self._get_dv_lines()
        if l_line is None:
            return fall_back

        line_with_entries = self._get_line_with_entries(l_line, alg_name)
        if line_with_entries is None:
            return fall_back

        nentries          = self._entries_from_line(line_with_entries)
        if nentries is None:
            return fall_back

        log.debug(f'Found {nentries} entries')

        return nentries
# ---------------------------------------------



'''
Module with functions used to test functions in decays/utilities.py
'''
import pytest

import ap_utilities.decays.utilities as aput

# --------------------------------------------------
class Data:
    '''
    Class used to store data needed by tests
    '''

    l_event_type = [
        '10000000',
        '10000010',
        '10000020',
        '10000021',
        '10000022',
        '10000023',
        '10000027',
        '10000030',
        '10002203',
        '10002213',
        '11100001',
        '11100003',
        '11100006',
        ]

    l_new_nick = [
            'Bd_Dmnpipl_eq_DPC',
            'Bd_Dmnpipl_eq_DPC',
            'Bd_Dstplenu_eq_PHSP_TC',
            'Bd_Dstplmunu_eq_PHSP_TC',
            'Bd_Kpimumu_eq_DPC',
            'Bd_Kpimumu_eq_DPC',
            'Bd_Kstee_eq_btosllball05_DPC',
            'Bd_Kstee_eq_btosllball05_DPC',
            'Bd_Kstee_eq_btosllball05_DPC',
            'Bd_Kstee_eq_btosllball05_DPC',
            'Bd_Kstee_eq_DPC',
            'Bd_Kstee_flatq2_eq_DPC_MomCut',
            'Bd_Ksteta_eplemng_eq_Dalitz_DPC',
            ]

    l_lower_case = [
            'data_24_magdown_24c1',
            'data_24_magdown_24c2',
            'data_24_magdown_24c3',
            'data_24_magdown_24c4',
            'data_24_magup_24c1',
            'data_24_magup_24c2',
            'data_24_magup_24c3',
            'data_24_magup_24c4',
            'bu_d0enu_kenu_eq_dpc_ptcut200mev_vismass4gev',
            'bd_denu_kstenu_eq_via_hvm_egdwc',
            'bs_dsenu_phienu_eq_dpc_hvm_egdwc',
            'bd_dmnpipl_eq_dpc',
            'bd_dmnpipl_eq_dpc',
            'bd_dstplenu_eq_phsp_tc',
            'bd_dstplmunu_eq_phsp_tc',
            'bd_kpimumu_eq_dpc',
            'bd_kpimumu_eq_dpc',
            'bu_kee_eq_btosllball05_dpc_spr',
            'bd_kstee_eq_btosllball05_dpc_ss',
            'bd_kstee_eq_btosllball05_dpc',
            'bd_kstee_eq_dpc',
            'bd_kstee_flatq2_eq_dpc_momcut',
            'bd_ksteta_eplemng_eq_dalitz_dpc',
            ]

    l_old_nick = [
            'Bd2DNuKstNuEE',
            'Bd2DPiEE',
            'Bd2DPiMM',
            'Bd2DstNuDPiKPiEE',
            'Bd2DstNuDPiKPiMM',
            'Bd2KPiEE',
            'Bd2KPiMM',
            'Bd2KstEE',
            'Bd2KstEE_central',
            'Bd2KstEE_high',
            'Bd2KstEE_low',
            'Bd2KstEEvNOFLT',
            'Bd2KstEEvPS',
            'Bd2KstEta_EEG',
            ]
# --------------------------------------------------
@pytest.mark.parametrize('event_type', Data.l_event_type)
def test_read_decay_name(event_type : str) -> None:
    '''
    Tests reading of decay name from YAML using event type
    '''
    name = aput.read_decay_name(event_type=event_type)

    print(f'{event_type:<20}{name:<50}')
# --------------------------------------------------
@pytest.mark.parametrize('new_nick', Data.l_new_nick)
def test_read_event_type(new_nick: str) -> None:
    '''
    Tests reading of event type from YAML using new_nick 
    '''
    event_type = aput.read_event_type(nickname=new_nick)
    print(event_type)
# --------------------------------------------------
@pytest.mark.parametrize('old_nick', Data.l_old_nick)
def test_new_from_old(old_nick : str) -> None:
    '''
    Will test function returning new nickname style
    from old nickname style
    '''
    old_nick = aput.new_from_old_nick(old_nick)
    print(old_nick)
# --------------------------------------------------
@pytest.mark.parametrize('new_nick', Data.l_new_nick)
def test_old_from_new(new_nick : str) -> None:
    '''
    Will test function returning old nickname style
    from new nickname style
    '''
    old_nick = aput.old_from_new_nick(new_nick)
    print(old_nick)
# --------------------------------------------------
@pytest.mark.parametrize('lower_case', Data.l_lower_case)
def test_name_from_lower_case(lower_case : str) -> None:
    '''
    Will test function that returns original sample name from lower
    case name
    '''
    name = aput.name_from_lower_case(lower_case)
    print(f'{lower_case:<40}{"->":<20}{name:<30}')
